import requests

from .utils import filter_list


class Size:
    """
    Instances are sized by combinations of CPU virtual cores, memory and disk space.
    Custom sizes can also be created by customers, as our quotas and therefore pricing are based on
    a combined allocation of CPU, RAM and disk.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/sizes'.format(api_url)

    def search(self, filter: str = None) -> dict:
        """
        Function to listing available instances sizes
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()



import requests

from .utils import filter_list


class Snapshots:
    """
    We provide a backup service for our Instances called snapshots. This takes an exact copy of the instance's
    virtual hard drive. At any point an instance can be restored to the state it was in when the snapshot was made.
    These snapshots can also be used to build a new instance to scale identically configured infrastructure.

    As snapshot storage is chargeable, at any time these can be deleted. They can also be scheduled rather than
    immediately created, and if desired repeated at the same schedule each week (although the repeated snapshot
    will overwrite itself each week not keep multiple weekly snapshots).
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/snapshots'.format(api_url)

    def create(self, name: str, instance_id: str, safe: str = 'false', cron_timing: str = None) -> dict:
        """
        Function to create a new or update an old snapshot
        :param instance_id: The ID of the instance to snapshot
        :param name: The name of the instance
        :param safe: If true the instance will be shut down during the snapshot to ensure all files are in a
                     consistent state (e.g. database tables aren't in the middle of being optimised and hence
                     risking corruption). The default is false so you experience no interruption of service,
                     but a small risk of corruption.
        :param cron_timing: If a valid cron string is passed, the snapshot will be saved as an automated snapshot,
                            continuing to automatically update based on the schedule of the cron sequence provided.
                            The default is nil meaning the snapshot will be saved as a one-off snapshot.
        :return: objects json
        """
        payload = {'instance_id': instance_id}

        if safe:
            payload['safe'] = safe

        if cron_timing:
            payload['cron_timing'] = cron_timing

        r = requests.put(self.url + '/{}'.format(name), headers=self.headers, params=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        Function to list snapshots
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def delete(self, name: str) -> dict:
        """
        Function to deleting a snapshot
        :param name: name of the instance
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(name), headers=self.headers)

        return r.json()



import os

from .charges import Charges
from .dns import Dns
from .firewall import Firewall
from .intances import Instances
from .kubernetes import Kubernetes
from .loadbalance import LoadBalance
from .networks import Networks
from .quota import Quota
from .regions import Regions
from .size import Size
from .snapshots import Snapshots
from .ssh import Ssh
from .templates import Templates
from .volumes import Volumes
from .webhook import WebHook


class Civo:

    def __init__(self, civo_token: str = None, api_url: str = None):
        """
        Init for Civo class
        :param civo_token: str, optional the token generate by civo
        """

        # Get token from env or pass to the class
        if not civo_token:
            self.token = os.getenv('CIVO_TOKEN', False)
        else:
            self.token = civo_token

        # Get api url from env or you can pass to the class
        if not api_url:
            self.api_url = os.getenv('CIVO_API', False)

        if self.api_url:
            self.api_url = api_url
        else:
            self.api_url = 'api.civo.com'

        # Create headers for all requests to civo api
        if not self.token:
            raise Exception('CIVO_TOKEN not found in the enviroment or is not declared in the class')

        self.headers = {'Authorization': 'bearer {}'.format(self.token)}

        # int all class
        self.ssh = Ssh(self.headers, self.api_url)
        self.instances = Instances(self.headers, self.api_url)
        self.networks = Networks(self.headers, self.api_url)
        self.snapshots = Snapshots(self.headers, self.api_url)
        self.volumes = Volumes(self.headers, self.api_url)
        self.firewalls = Firewall(self.headers, self.api_url)
        self.dns = Dns(self.headers, self.api_url)
        self.loadbalance = LoadBalance(self.headers, self.api_url)
        self.webhook = WebHook(self.headers, self.api_url)
        self.size = Size(self.headers, self.api_url)
        self.regions = Regions(self.headers, self.api_url)
        self.templates = Templates(self.headers, self.api_url)
        self.quota = Quota(self.headers, self.api_url)
        self.charges = Charges(self.headers, self.api_url)
        self.kubernetes = Kubernetes(self.headers, self.api_url)



import requests

from .utils import filter_list


class Networks:
    """
    To manage the private networks for an account, there are a set of APIs for listing them, as well as adding,
    renaming and removing them by ID.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/networks'.format(api_url)

    def create(self, label: str) -> dict:
        """
        Function to create a private network
        :param label: a string that will be the displayed name/reference for the network.
        :return: object json
        """
        payload = {'label': label}
        r = requests.post(self.url, headers=self.headers, params=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        Function to listing the private networks
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: [
                  {
                    "id": "50f2fffa-f81e-4e96-830f-e78f7e565e6f",
                    "name": "example-ltd-a775-development-75362452-562f-4b70-a65a-aeb4d4cd6864",
                    "region": "lon1",
                    "default": false,
                    "cidr": "0.0.0.0/0",
                    "label": "development"
                  }
                ]
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def rename(self, id: str, label: str) -> dict:
        """
        Function to renaming a network
        :param id: id of the objects
        :param label: the new label to use.
        :return: object json
        """
        payload = {'label': label}
        r = requests.put(self.url + '/{}'.format(id), headers=self.headers, params=payload)

        return r.json()

    def delete(self, id: str) -> dict:
        """
        Function to removing a private network
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()



import requests

from .utils import filter_list


class Kubernetes:
    """
    Kubernetes clusters are a number of instances on the Civo cloud platform running the Kubernetes cloud orchestration platform.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/kubernetes/clusters'.format(api_url)
        self.kube_version = 'https://{}/v2/kubernetes/versions'.format(api_url)
        self.marketplace_url = 'https://{}/v2/kubernetes/applications'.format(api_url)

    def create(self, name: str, num_nodes: int = 3, nodes_size: str = 'g2.small', kubernetes_version: str = None, tags: str = None) -> dict:
        """
        Function to create a cluster of kubernetes
        :param name: a name for your cluster, must be unique within your account (required)
        :param num_nodes: the number of instances to create (optional, the default at the time of writing is 3)
        :param nodes_size: the size of each node (optional, the default is currently g2.small)
        :param kubernetes_version: the version of k3s to install (optional, the default is currently the latest available)
        :param tags: a space separated list of tags, to be used freely as required (optional)
        :return: dict
        """
        payload = {'name': name, 'num_target_nodes': num_nodes, 'target_nodes_size': nodes_size}

        if tags:
            payload['tags'] = tags

        if kubernetes_version:
            payload['kubernetes_version'] = kubernetes_version

        r = requests.post(self.url, headers=self.headers, params=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        A list of clusters accessible from an account is available
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
           you can filter by any object that is inside the json
        :return: objects dict
        """
        payload = {}

        r = requests.get(self.url, headers=self.headers, params=payload)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def retrieving(self, id: str) -> object:
        """
        Function to retrieving a single cluster's details
        :param id: id of the objects
        :return: object json
        """
        r = requests.get(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()

    def update(self, id: str, name: str = None, num_nodes: int = None, applications: str = None) -> dict:
        """
        Function to update a cluster of kubernetes
        :param applications: a comma separated list of applications to install. Spaces within application names are fine, but shouldn't be either side of the comma.
        :param id: id of the cluster
        :param name: the cluster's new name
        :param num_nodes: how many nodes should the cluster scale to.
        :return: dict
        """
        payload = {}

        if name:
            payload['name'] = name

        if num_nodes:
            payload['num_target_nodes'] = num_nodes

        if applications:
            payload['applications'] = applications

        r = requests.put(self.url + '/{}'.format(id), headers=self.headers, params=payload)

        return r.json()

    def marketplace(self, filter: str = None) -> dict:
        """
        A user can install applications in to their cluster from the marketplace using the `update` call above.
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
           you can filter by any object that is inside the json
        :return: objects dict
        """
        payload = {}

        r = requests.get(self.marketplace_url, headers=self.headers, params=payload)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def delete(self, id: str) -> dict:
        """
        A user can delete a cluster and all underlying nodes.
        :param id: id of the cluster
        :return: dict
        """

        r = requests.delete(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()

    def recycle(self, id: str, hostname: str) -> dict:
        """
        A user can delete and recreate one of the underlying nodes, if it's having a problem.
        :param hostname: he name of the node to recycle.
        :param id: id of the cluster
        :return: dict
        """
        payload = {'hostname': hostname}

        r = requests.post(self.url + '/{}/recycle'.format(id), headers=self.headers, payload=payload)

        return r.json()

    def versions(self, filter: str = None) -> dict:
        """
        A list of versions available to install isavailable
        :param filter: Filter json object the format is 'type:stable',
           you can filter by any object that is inside the json
        :return: objects dict
        """
        payload = {}

        r = requests.get(self.kube_version, headers=self.headers, params=payload)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()



import requests

from .utils import filter_list


class Instances:
    """
    Instances are running virtual servers on the Civo cloud platform. They can be of variable size.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/instances'.format(api_url)

    def create(self, hostname: str, size: str, template_id: str, reverse_dns: str = None, region: str = None,
               public_ip: str = 'create', move_ip_from: str = None, count: int = 1, network_id: str = None,
               snapshot_id: str = None, initial_user: str = None, ssh_key_id: str = None, script: str = None, tags: str = None) -> dict:
        # TODO: Pending check
        """
        Function to create instance
        :param hostname: a fully qualified domain name that should be set as the instance's hostname (required)
        :param size: the identifier for the size, from the current list (required)
        :param template_id: the ID for the template to use to build the instance, from the current templates.
               Parameter also accepted as template. (optional; but must be specified if no snapshot is specified)
        :param reverse_dns: a fully qualified domain name that should be used as the instance's IP's reverse DNS (optional, uses the hostname if unspecified)
        :param region: the identifier for the region, from the current region (optional; a random one will be picked by the system)
        :param public_ip: this should be either none, create or from. If from is specified then the move_ip_from
               parameter should also be specified (and contain the ID of the instance that will be releasing its IP).
               As aliases true will be treated the same as create and false will be treated the same as none.
               If create or true is specified it will automatically allocate an initial public IP address,
               rather than having to add the first one later (optional; default is create)
        :param move_ip_from: parameter should also be specified (and contain the ID of the instance that will be releasing its IP).
        :param count: the number of instances to create (optional, default is 1)
        :param network_id: this must be the ID of the network from the network listing (optional;
               default network used when not specified)
        :param snapshot_id: the ID for the snapshot to use to build the instance, from your snapshots
               (optional; but must be specified if no template is specified)
        :param initial_user: the name of the initial user created on the server
               (optional; this will default to the template's default_username and fallback to "civo")
        :param ssh_key_id: the ID of an already uploaded SSH public key to use for login to the default user
               (optional; if one isn't provided a random password will be set and returned in the initial_password field)
        :param script: the contents of a script that will be uploaded to /usr/local/bin/civo-user-init-script on your
               instance, read/write/executable only by root and then will be
               executed at the end of the cloud initialization
        :param tags: a space separated list of tags, to be used freely as required (optional)
        :return: objects json
        """

        payload = {'hostname': hostname, 'size': size, 'template_id': template_id}

        if reverse_dns:
            payload['reverse_dns'] = reverse_dns

        if region:
            payload['region'] = region

        if public_ip:
            payload['public_ip'] = public_ip

        if move_ip_from:
            payload['move_ip_from'] = move_ip_from

        if count:
            payload['count'] = count

        if network_id:
            payload['network_id'] = network_id

        if snapshot_id:
            payload['snapshot_id'] = snapshot_id

        if ssh_key_id:
            payload['ssh_key_id'] = ssh_key_id

        if script:
            payload['script'] = script

        if initial_user:
            payload['initial_user'] = initial_user

        if tags:
            payload['tags'] = tags

        r = requests.post(self.url, headers=self.headers, params=payload)

        return r.json()

    def search(self, tags: str = None, page: str = None, per_page: str = None, filter: str = None) -> dict:
        """
        Function to list all instances
        :param tags: a space separated list of tags, to be used freely as required.
               If multiple are supplied, instances must much all tags to be returned (not one or more)
        :param page: which page of results to return (defaults to 1)
        :param per_page: how many instances to return per page (defaults to 20)
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
           you can filter by any object that is inside the json
        :return: objects json
        """
        payload = {}

        if tags:
            payload['tags'] = tags

        if page:
            payload['page'] = page

        if per_page:
            payload['per_page'] = per_page

        r = requests.get(self.url, headers=self.headers, params=payload)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def retrieving(self, id: str) -> object:
        """
        Function to retrieving a single instance
        :param id: id of the objects
        :return: object json
        """
        r = requests.get(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()

    def retagging(self, id: str, tags: str) -> dict:
        """
        Function to retagging an instance
        :param id: id of the objects
        :param tags: a space separated list of tags, to be used freely as required (optional)
        :return: object json
        """
        payload = {'tags': tags}

        r = requests.put(self.url + '/{}/tags'.format(id), headers=self.headers, params=payload)

        return r.json()

    def rebooting(self, id: str, type_reboot: str) -> dict:
        """
        Function to rebooting an instance
        :param id: id of the objects
        :param type_reboot: (reboots|hard_reboots|soft_reboots)
        :return: object json
        """
        r = requests.post(self.url + '/{}/{}'.format(id, type_reboot), headers=self.headers)

        return r.json()

    def stop(self, id: str) -> dict:
        """
        Function to shutting down an instance
        :param id: id of the objects
        :return: object json
        """
        r = requests.put(self.url + '/{}/stop'.format(id), headers=self.headers)

        return r.json()

    def start(self, id: str) -> dict:
        """
        Function to starting an instance after being shut down
        :param id: id of the objects
        :return: object json
        """
        r = requests.put(self.url + '/{}/start'.format(id), headers=self.headers)

        return r.json()

    def resizing(self, id: str, size: str) -> dict:
        """
        Function to upgrading (resizing) an instance
        :param id: id of the objects
        :param size: the identifier for the size, from the current list
        :return: object json
        """
        payload = {'size': size}

        r = requests.put(self.url + '/{}/resize'.format(id), headers=self.headers, params=payload)

        return r.json()

    def firewall(self, id: str, firewall_id: str = None) -> dict:
        """
        Function to setting the firewall for an instance
        :param id: id of the objects
        :param firewall_id: the ID of the firewall to use, from the current list. If left blank or not sent,
               the default firewall will be used (open to all)
        :return: object json
        """
        payload = {}

        if firewall_id:
            payload['firewall_id'] = firewall_id

        r = requests.put(self.url + '/{}/firewall'.format(id), headers=self.headers, params=payload)

        return r.json()

    def moving_ip(self, id: str, ip: str) -> dict:
        """
        Function to moving a public IP between instances
        :param id: id of the objects
        :param ip: ip address
        :return: object json
        """
        r = requests.put(self.url + '/{}/ip/{}'.format(id, ip), headers=self.headers)

        return r.json()

    def delete(self, id: str) -> dict:
        """
        Function to deleting an instance
        :param id: id of the objects
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()


import requests

from .utils import filter_list


class Ssh:
    """
    To manage the SSH keys for an account that are used for logging in to instances,
    there are a set of APIs for listing the SSH public keys currently stored,
    as well as adding and removing them by name.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/sshkeys'.format(api_url)

    def create(self, name: str, public_key: str) -> dict:
        """
        Function to uploading a SSH public key
        :param name: Name of the key
        :param public_key: Public key of the ssh
        :return: object json
        """
        payload = {'name': name, 'public_key': public_key}
        r = requests.post(self.url, headers=self.headers, params=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        Function to listing the SSH public keys
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def retrieving(self, id: str) -> dict:
        """
        Function to retrieving a SSH key
        :param id: id of the objects
        :return: object json
        """
        r = requests.get(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()

    def updating(self, id: str, name: str) -> dict:
        """
        Function to updating a SSH key
        :param id: id of the objects
        :param name: name to change
        :return: object json
        """
        payload = {'name': name}
        r = requests.put(self.url + '/{}'.format(id), headers=self.headers, params=payload)

        return r.json()

    def delete(self, id: str) -> dict:
        """
        Function to removing a SSH key
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()



from datetime import datetime

import requests


class Charges:
    """
    The system tracks usage of paid service on an hourly basis.
    It doesn't track how much to charge for any particular product, but it will report for each instance,
    IP address and snapshot the amount of hours it's in use for.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/charges'.format(api_url)

    def get(self, date_from: str = None, date_to: str = None) -> dict:
        """
        Function to listing charges
        :param date_from: The from date like '2019-07-01'
        :param date_to: The to date like '2019-07-30'
        :return: object json
        """
        payload = {}

        if date_from:
            date = datetime.strptime(date_from, '%Y-%m-%d')
            payload['from'] = date.astimezone().isoformat()

        if date_to:
            date = datetime.strptime(date_to, '%Y-%m-%d')
            payload['to'] = date.astimezone().isoformat()

        r = requests.post(self.url, headers=self.headers, params=payload)

        return r.json()



import requests

from .utils import filter_list


class LoadBalance:
    """
    If you want to create a load balancer for your instances, to spread your web traffic
    between them then you can easily launch a managed load balancer service on Civo.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/loadbalancers'.format(api_url)

    def create(self, backends: list, hostname: str = None, tls_certificate: str = None,
               tls_key: str = None,
               max_request_size: int = 20, policy: str = 'random',
               health_check_path: str = '/', fail_timeout: str = '30', max_conns: str = '10',
               ignore_invalid_backend_tls: str = 'true'):
        """
        Function to setup a new load balancer
        :param hostname: the hostname to receive traffic for, e.g. "www.example.com"
                         (optional: sets hostname to loadbalancer-uuid.civo.com if blank)
        :param backends: a list of backend instances, each containing an instance_id, protocol (http or https) and port
                         ej: [{'instance_id': '8878789', 'protocol': 'https', 'port': 443}, {'instance_id': '123234234', 'protocol': 'https', 'port': 443}]
        :param tls_certificate: if your protocol is https then you should send the TLS certificate in Base64-encoded PEM format
        :param tls_key: if your protocol is https then you should send the TLS private key in Base64-encoded PEM format
        :param max_request_size: the size in megabytes of the maximum request content that will be accepted, defaults to 20
        :param policy: one of: least_conn (sends new requests to the least busy server), random (sends new requests
                       to a random backend), round_robin (sends new requests to the next backend in order),
                       ip_hash (sends requests from a given IP address to the same backend), default is "random"
        :param health_check_path: what URL should be used on the backends to determine if it's OK (2xx/3xx status), defaults to "/"
        :param fail_timeout: how long to wait in seconds before determining a backend has failed, defaults to 30
        :param max_conns: how many concurrent connections can each backend handle, defaults to 10
        :param ignore_invalid_backend_tls: should self-signed/invalid certificates be ignored from the backend servers, defaults to true
        :return: object json
        """

        payload = {}

        for number, backend in enumerate(backends):
            for value in backend:
                payload['backends[{}][{}]'.format(number, value)] = backend[value]

        if hostname:
            payload['hostname'] = hostname

        if tls_certificate:
            payload['tls_certificate'] = tls_certificate

        if tls_key:
            payload['tls_key'] = tls_key

        if max_request_size:
            payload['max_request_size'] = max_request_size

        if policy:
            payload['policy'] = policy

        if health_check_path:
            payload['health_check_path'] = health_check_path

        if fail_timeout:
            payload['fail_timeout'] = fail_timeout

        if max_conns:
            payload['max_conns'] = max_conns

        if ignore_invalid_backend_tls:
            payload['ignore_invalid_backend_tls'] = ignore_invalid_backend_tls

        r = requests.post(self.url, headers=self.headers, data=payload)

        return r.json()

    def update(self, id: str, backends: list, hostname: str = None, tls_certificate: str = None,
               tls_key: str = None,
               max_request_size: int = 20, policy: str = 'random',
               health_check_path: str = '/', fail_timeout: str = '30', max_conns: str = '10',
               ignore_invalid_backend_tls: str = 'true'):
        """
        Function to setup a new load balancer
        :param hostname: the hostname to receive traffic for, e.g. "www.example.com"
                         (optional: sets hostname to loadbalancer-uuid.civo.com if blank)
        :param backends: a list of backend instances, each containing an instance_id, protocol (http or https) and port
                         ej: [{'instance_id': '8878789', 'protocol': 'https', 'port': 443}, {'instance_id': '123234234', 'protocol': 'https', 'port': 443}]
        :param tls_certificate: if your protocol is https then you should send the TLS certificate in Base64-encoded PEM format
        :param tls_key: if your protocol is https then you should send the TLS private key in Base64-encoded PEM format
        :param max_request_size: the size in megabytes of the maximum request content that will be accepted, defaults to 20
        :param policy: one of: least_conn (sends new requests to the least busy server), random (sends new requests
                       to a random backend), round_robin (sends new requests to the next backend in order),
                       ip_hash (sends requests from a given IP address to the same backend), default is "random"
        :param health_check_path: what URL should be used on the backends to determine if it's OK (2xx/3xx status), defaults to "/"
        :param fail_timeout: how long to wait in seconds before determining a backend has failed, defaults to 30
        :param max_conns: how many concurrent connections can each backend handle, defaults to 10
        :param ignore_invalid_backend_tls: should self-signed/invalid certificates be ignored from the backend servers, defaults to true
        :return: object json
        """

        payload = {}

        for number, backend in enumerate(backends):
            for value in backend:
                payload['backends[{}][{}]'.format(number, value)] = backend[value]

        if hostname:
            payload['hostname'] = hostname

        if tls_certificate:
            payload['tls_certificate'] = tls_certificate

        if tls_key:
            payload['tls_key'] = tls_key

        if max_request_size:
            payload['max_request_size'] = max_request_size

        if policy:
            payload['policy'] = policy

        if health_check_path:
            payload['health_check_path'] = health_check_path

        if fail_timeout:
            payload['fail_timeout'] = fail_timeout

        if max_conns:
            payload['max_conns'] = max_conns

        if ignore_invalid_backend_tls:
            payload['ignore_invalid_backend_tls'] = ignore_invalid_backend_tls

        r = requests.put(self.url + '/{}'.format(id), headers=self.headers, data=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        Function to list load balancers
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def delete(self, id: str) -> dict:
        """
        Function to deleting a load balancer
        :param id: ID of the load balancer to delete
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()



import requests

from .utils import filter_list


class Volumes:
    """
    We provide a flexible size additional storage service for our Instances called volumes.
    This creates and attaches an additional virtual disk to the instance, allowing you to put backups or database
    files on the separate volume and later move the volume to another instance.

    As volume storage is chargeable, at any time these can be deleted.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/volumes'.format(api_url)

    def create(self, name: str, size_gb: str, bootable: str = 'false') -> dict:
        """
        Function to create a new volume
        :param name: A name that you wish to use to refer to this volume (required)
        :param size_gb: A minimum of 1 and a maximum of your available disk space from your quota specifies
                        the size of the volume in gigabytes (required).
        :param bootable: Mark the volume as bootable with a boolean (optional; defaults to false).
        :return: objects json
        """
        payload = {'name': name, 'size_gb': size_gb}

        if bootable:
            payload['bootable'] = bootable

        r = requests.post(self.url, headers=self.headers, params=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        Function to list volumes
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def resizing(self, id: str, size_gb: str) -> dict:
        """
        Function to resizing a volume
        :param id: id of the objects
        :param size_gb: A minimum of the existing size of the volume plus 1 and a maximum of your available
                        disk space from your quota specifies the size of the volume in gigabytes (required).
        :return: object json
        """
        payload = {'size_gb': size_gb}

        r = requests.put(self.url + '/{}/resize'.format(id), headers=self.headers, params=payload)

        return r.json()

    def attach(self, id: str, instance_id: str) -> dict:
        """
        Function to attach a volume to an instance
        :param id: id of the objects
        :param instance_id: The ID of an instance that you wish to attach this volume to (required)
        :return: object json
        """
        payload = {'instance_id': instance_id}

        r = requests.put(self.url + '/{}/attach'.format(id), headers=self.headers, params=payload)

        return r.json()

    def detach(self, id: str) -> dict:
        """
        Function to detach a volume from an instance
        :param id: id of the objects
        :return: object json
        """
        r = requests.put(self.url + '/{}/detach'.format(id), headers=self.headers)

        return r.json()

    def delete(self, id: str) -> dict:
        """
        Function to deleting a volume
        :param id: name of the instance
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()


import requests

from .utils import filter_list


class Dns:
    """
    We host reverse DNS for all instances automatically. If you'd like to manage forward (normal)
    DNS for your domains, you can do that for free within your account. This API is effectively split
    in to two parts: 1) Managing domain names themselves, and 2) Managing records within those domain names.
    We don't offer registration of domains names, this is purely for hosting the DNS.
    If you're looking to buy a domain name, we recommend LCN.com for their excellent friendly support
    and very competitive prices.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/dns'.format(api_url)

    def create(self, name: str) -> dict:
        """
        Function to setup a new domain
        :param name: the domain name, e.g. "example.com"
        :return: object json
        """
        payload = {'name': name}
        r = requests.post(self.url, headers=self.headers, params=payload)

        return r.json()

    def update(self, id: str, name: str) -> dict:
        """
        Function to update a new domain
        :param name: the domain name, e.g. "example.com"
        :return: object json
        """
        payload = {'name': name}
        r = requests.put(self.url + '/{}'.format(id), headers=self.headers, params=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        Function to list all dns domains
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def delete(self, id: str) -> dict:
        """
        Function to deleting a domain
        :param id: id of the domain object
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()

    def create_record(self, id: str, type: str, name: str, value: str, priority: str, ttl: str = '600') -> dict:
        """
        Function to create a new DNS record
        :param id: id of the domain object
        :param type: the choice of RR type from (a, cname, mx or txt)
        :param name: the portion before the domain name (e.g. www) or an @ for the apex/root domain
                     (you cannot use an A record with an amex/root domain)
        :param value: the IP address (A or MX), hostname (CNAME or MX) or text value (TXT) to serve for this record
        :param priority: useful for MX records only, the priority mail should be attempted it (defaults to 10)
        :param ttl: how long caching DNS servers should cache this record for, in seconds
                    (the minimum is 600 and the default if unspecified is 600)
        :return: object json
        """
        payload = {'type': type, 'name': name, 'value': value, 'priority': priority, 'ttl': ttl}
        r = requests.post(self.url + '{}/records'.format(id), headers=self.headers, params=payload)

        return r.json()

    def update_record(self, id: str, id_record: str, type: str = None, name: str = None, value: str = None,
                      priority: str = None,
                      ttl: str = None) -> dict:
        """
        Function to update a DNS record
        :param id: id of the domain object
        :param id_record: id of the record domain object
        :param type: the choice of RR type from (a, cname, mx or txt)
        :param name: the portion before the domain name (e.g. www) or an @ for the apex/root domain
                     (you cannot use an A record with an amex/root domain)
        :param value: the IP address (A or MX), hostname (CNAME or MX) or text value (TXT) to serve for this record
        :param priority: useful for MX records only, the priority mail should be attempted it (defaults to 10)
        :param ttl: how long caching DNS servers should cache this record for, in seconds
                    (the minimum is 600 and the default if unspecified is 600)
        :return: object json
        """
        payload = {}

        if type:
            payload['type'] = type

        if name:
            payload['name'] = name

        if value:
            payload['value'] = value

        if priority:
            payload['priority'] = priority

        if ttl:
            payload['ttl'] = ttl

        r = requests.put(self.url + '{}/records/{}'.format(id, id_record), headers=self.headers, params=payload)

        return r.json()

    def lists_record(self, id: str, filter: str = None) -> dict:
        """
        Function to list DNS records
        :param id: Firewall object id
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url + '{}/records'.format(id), headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def delete_record(self, id: str, record_id: str) -> dict:
        """
        Function to deleting a dns record
        :param id: id of the dns object
        :param record_id: id of the dns record object
        :return: object json
        """
        r = requests.delete(self.url + '/{}/records/{}'.format(id, record_id), headers=self.headers)

        return r.json()



import requests


class Quota:
    """
    Our quotas (and therefore our pricing), are based on a combined allocation of CPU, RAM and disk.
    All customers start on a basic quota level and after a period of proving that the quota is being
    handled correctly or after a call to our offices, we can increase this quota.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/quota'.format(api_url)

    def get(self) -> dict:
        """
        Function to get quota
        :return: object json
        """
        r = requests.post(self.url, headers=self.headers)

        return r.json()



import requests

from .utils import filter_list


class WebHook:
    """
    Open certain actions taking place within your account, we can trigger a JSON POST callback to a URL or your choice.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/webhooks'.format(api_url)

    def create(self, events: str, url: str, secret: str = None) -> dict:
        """
        Function to create a new webhook
        :param events: This is a list of events that the webhook should be triggered for.
                       Alternative you can use a list containing a single entry of "*" to signify trigger for all events. (required)
        :param url: This is the URL to send the webhook to (required).
        :param secret: This is if you want to specify your own secret, if not a random one will be created for you (optional).
        :return: object json
        """
        payload = {'events': events, 'url': url}

        if secret:
            payload['secret'] = secret

        r = requests.post(self.url, headers=self.headers, params=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        Function to listing webhooks
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """

        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def test(self, id: str) -> dict:
        """
        Function to test a webhook
        :param id: id of webhook object
        :return: object json
        """
        r = requests.post(self.url + '/{}/test'.format(id), headers=self.headers)

        return r.json()

    def update(self, id: str, events: str, url: str, secret: str = None) -> dict:
        """
        Function to update a new webhook
        :param id: id of the webhook object
        :param events: This is a list of events that the webhook should be triggered for.
                       Alternative you can use a list containing a single entry of "*" to signify trigger for all events. (required)
        :param url: This is the URL to send the webhook to (required).
        :param secret: This is if you want to specify your own secret, if not a random one will be created for you (optional).
        :return: object json
        """
        payload = {'events': events, 'url': url}

        if secret:
            payload['secret'] = secret

        r = requests.put(self.url + '/{}'.format(id), headers=self.headers, params=payload)

        return r.json()

    def delete(self, id: str) -> dict:
        """
        Function to delete a new webhook
        :param id: id of the webhook object
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()



import requests

from .utils import filter_list


class Templates:
    """
    Instances are built from a template. Templates may be either a base OS install such as Ubuntu 14.04 LTS,
    a control-panel based hosting setup or a fully setup application ready to configure for your use.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/templates'.format(api_url)

    def create(self, code: str, id: str = None, name: str = None, volume_id: str = None, image_id: str = None,
               short_description: str = None, description: str = None, default_username: str = None,
               cloud_config: str = None) -> dict:
        """
        Function to create a new template
        :param id: This is a short identifier for the template, it should be lowercase letters, dashs, underscores,
                   full stop/periods and numbers only (optional: defaults to a new UUID).
        :param code: This is a unqiue, alphanumerical, short, human readable code for the template (required).
        :param name: This is a short human readable name for the template (optional).
        :param volume_id: This is the ID of a bootable volume, either owned by you or global
                          (optional; but must be specified if no image_id is specified).
        :param image_id: This is the Openstack Glance Image ID or the ID of another template,
                         either owned by you or global (optional; but must be specified if no volume_id is specified).
        :param short_description: A one line description of the template (optional)
        :param description: A multi-line description of the template, in Markdown format (optional).
        :param default_username: he default username to suggest that the user creates (optional: defaults to civo).
        :param cloud_config: Commonly referred to as 'user-data', this is a customisation script that is run after
                             the instance is first booted. We recommend using cloud-config as it's a great
                             distribution-agnostic way of configuring cloud servers.
                             If you put $INITIAL_USER in your script, this will automatically be replaced
                             by the initial user chosen when creating the instance, $INITIAL_PASSWORD will be
                             replaced with the random password generated by the system, $HOSTNAME is the fully
                             qualified domain name of the instance and $SSH_KEY
                             will be the content of the SSH public key.
                             (this is technically optional, but you won't really be able to use instances without it
                             see our learn guide on templates for more information)
        :return: object json
        """
        payload = {'code': code}

        if id:
            payload['id'] = id

        if name:
            payload['name'] = name

        if volume_id:
            payload['volume_id'] = volume_id

        if image_id:
            payload['image_id'] = image_id

        if short_description:
            payload['short_description'] = short_description

        if description:
            payload['description'] = description

        if default_username:
            payload['default_username'] = default_username

        if cloud_config:
            files = {'cloud_config': open(cloud_config, 'rb')}
            r = requests.get(self.url, headers=self.headers, files=files, params=payload)
        else:
            r = requests.get(self.url, headers=self.headers, params=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        Function to listing available templates
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def update(self, template_id: str, code: str, id: str = None, name: str = None, volume_id: str = None, image_id: str = None,
               short_description: str = None, description: str = None, default_username: str = None,
               cloud_config: str = None):
        """
        Function to create a new template
        :param template_id: id of template to update
        :param id: This is a short identifier for the template, it should be lowercase letters, dashs, underscores,
                   full stop/periods and numbers only (optional: defaults to a new UUID).
        :param code: This is a unqiue, alphanumerical, short, human readable code for the template (required).
        :param name: This is a short human readable name for the template (optional).
        :param volume_id: This is the ID of a bootable volume, either owned by you or global
                          (optional; but must be specified if no image_id is specified).
        :param image_id: This is the Openstack Glance Image ID or the ID of another template,
                         either owned by you or global (optional; but must be specified if no volume_id is specified).
        :param short_description: A one line description of the template (optional)
        :param description: A multi-line description of the template, in Markdown format (optional).
        :param default_username: he default username to suggest that the user creates (optional: defaults to civo).
        :param cloud_config: Commonly referred to as 'user-data', this is a customisation script that is run after
                             the instance is first booted. We recommend using cloud-config as it's a great
                             distribution-agnostic way of configuring cloud servers.
                             If you put $INITIAL_USER in your script, this will automatically be replaced
                             by the initial user chosen when creating the instance, $INITIAL_PASSWORD will be
                             replaced with the random password generated by the system, $HOSTNAME is the fully
                             qualified domain name of the instance and $SSH_KEY
                             will be the content of the SSH public key.
                             (this is technically optional, but you won't really be able to use instances without it
                             see our learn guide on templates for more information)
        :return: object json
        """
        payload = {'code': code}

        if id:
            payload['id'] = id

        if name:
            payload['name'] = name

        if volume_id:
            payload['volume_id'] = volume_id

        if image_id:
            payload['image_id'] = image_id

        if short_description:
            payload['short_description'] = short_description

        if description:
            payload['description'] = description

        if default_username:
            payload['default_username'] = default_username

        if cloud_config:
            files = {'cloud_config': open(cloud_config, 'rb')}
            r = requests.put(self.url + '/{}'.format(template_id), headers=self.headers, files=files, params=payload)
        else:
            r = requests.put(self.url + '/{}'.format(template_id), headers=self.headers, params=payload)

        return r.json()

    def delete(self, template_id: str) -> dict:
        """
        Function to deleting a template
        :param template_id: id of template to delete
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(template_id), headers=self.headers)

        return r.json()


import requests

from .utils import filter_list


class Firewall:
    """
    The simplest solution for most customers is to configure a firewall within their
    Instances using either iptables which is powerful or Uncomplicated Firewall/ufw
    which is much simpler but only works on Ubuntu.

    As an another option, customers can configure custom firewall rules for their
    instances using the Firewall API which adjusts the security group for your network
    of instances. These are a freely configurable option, however customers
    should be careful to not lock out their access to the instances.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/firewalls'.format(api_url)

    def create(self, name: str) -> dict:
        """
        Function to create a new firewall
        :param name: A unique name for this firewall within your account
        :return: object json
        """
        payload = {'name': name}
        r = requests.post(self.url, headers=self.headers, params=payload)

        return r.json()

    def search(self, filter: str = None) -> dict:
        """
        Function to list firewalls
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def delete(self, id: str) -> dict:
        """
        Function to deleting a firewall
        :param id: id of the firewall object
        :return: object json
        """
        r = requests.delete(self.url + '/{}'.format(id), headers=self.headers)

        return r.json()

    def create_rule(self, id: str, start_port: str, protocol: str = 'tcp', end_port: str = None,
                    cidr: str = '0.0.0.0/0',
                    direction: str = 'inbound', label: str = None) -> dict:
        """
        An account holder can create firewall rules for a specific firewall, but there is a quota'd limit
        to the number of rules that can be created, but generally this is much higher than most customers
        will require and it can be increased if required.
        :param id: Firewall id object
        :param protocol: the protocol choice from tcp, udp or icmp (the default if unspecified is tcp)
        :param start_port: the start of the port range to configure for this rule (or the single port if required)
        :param end_port: the end of the port range (this is optional, by default it will only apply to the single
               port listed in start_port)
        :param cidr: the IP address of the other end (i.e. not your instance) to affect, or a valid network CIDR
                     (defaults to being globally applied, i.e. 0.0.0.0/0)
        :param direction: will this rule affect inbound or outbound traffic (by default this is inbound)
        :param label: a string that will be the displayed name/reference for this rule (optional)
        :return: object json
        """
        payload = {'protocol': protocol, 'start_port': start_port, 'cidr': cidr, 'direction': direction}

        if end_port:
            payload['end_port'] = end_port

        if label:
            payload['label'] = label

        r = requests.post(self.url + '/{}/rules'.format(id), headers=self.headers, params=payload)

        return r.json()

    def lists_rule(self, id: str, filter: str = None) -> dict:
        """
        Function to list firewalls rules
        :param id: Firewall id object
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url + '/{}/rules'.format(id), headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()

    def delete_rule(self, id: str, rule_id: str) -> dict:
        """
        Function to deleting a firewall rule
        :param id: id of the firewall object
        :param rule_id: id of the firewall rule object
        :return: object json
        """
        r = requests.delete(self.url + '/{}/rules/{}'.format(id, rule_id), headers=self.headers)

        return r.json()


"""
File to handle all utils
"""


def filter_list(data: dict, filter: str) -> list:
    """
    Function to handle filter in all list of the api
    :param data: object json
    :param searchs: the filter in this format (label:Home, id:6224cd2b-d416-4e92-bdbb-db60521c8eb9)
    :return: the found object in json format
    """
    filter_split = filter.split(':')
    search_in = filter_split[0]
    search = filter_split[1]

    try:
        search = int(search)
    except ValueError:
        pass

    try:
        data_search = data['items']
    except TypeError:
        data_search = data

    return [element for element in data_search if element[search_in] == search]



import requests

from .utils import filter_list


class Regions:
    """
    Civo will be hosted in multiple datacentres (a.k.a. regions), with more coming online all the time.
    You can choose when creating an instance which region to have it hosted in (necessary if you want to
    share a private network between your instances) - or you can leave it for Civo to allocate you to a
    region if you don't care.
    """

    def __init__(self, headers, api_url):
        self.headers = headers
        self.url = 'https://{}/v2/regions'.format(api_url)

    def search(self, filter: str = None) -> dict:
        """
        Function to listing available regions
        :param filter: Filter json object the format is 'id:6224cd2b-d416-4e92-bdbb-db60521c8eb9',
                       you can filter by any object that is inside the json
        :return: object json
        """
        r = requests.get(self.url, headers=self.headers)

        if filter:
            data = r.json()
            return filter_list(data=data, filter=filter)

        return r.json()
